package com.rjconsultores.ventaboletos.web.gui.controladores.configuracioneccomerciales;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Button;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Decimalbox;
import org.zkoss.zul.Longbox;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Textbox;

import com.rjconsultores.ventaboletos.entidad.Parada;
import com.rjconsultores.ventaboletos.entidad.Transportadora;
import com.rjconsultores.ventaboletos.entidad.Voucher;
import com.rjconsultores.ventaboletos.exception.BusinessException;
import com.rjconsultores.ventaboletos.service.ParadaService;
import com.rjconsultores.ventaboletos.service.TransportadoraService;
import com.rjconsultores.ventaboletos.service.VoucherService;
import com.rjconsultores.ventaboletos.vo.configuracioneccomerciales.VoucherVO;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderPadrao;

import lombok.Getter;
import lombok.Setter;

@Getter
@Setter
@Controller("legalizacaoMassivaController")
@Scope("prototype")
public class LegalizacaoMassivaController extends MyGenericForwardComposer {

	private static Logger log = LogManager.getLogger(LegalizacaoMassivaController.class); 
	private static final long serialVersionUID = 1L;
	private static final String TITULO = "legalizacaoMassivaController.window.title";
	
	@Autowired
	private VoucherService voucherService;
	
	@Autowired
	private ParadaService paradaService;
	
	@Autowired
	private TransportadoraService transportadoraService;
	
	private MyListbox voucherList;
	private Paging pagingLegalizar;
	private Longbox txtNumInicial;
	private Longbox txtNumFinal;
	private Textbox txtNumContrato;
	private Textbox txtNit;
	private Textbox txtNome;
	private Combobox cmbOrigem;
	private Combobox cmbDestino;
	private Button btnLegalizar;
	private Combobox cmbTransportadora;
	private Decimalbox txtValorLegalizado;
	
	private List<Transportadora> lsTransportadora;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		
		setLsTransportadora(transportadoraService.obtenerTodos());
		
		super.doAfterCompose(comp);

		voucherList.setItemRenderer(new RenderPadrao<VoucherVO>(VoucherVO.class));
		voucherList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				VoucherVO cc = (VoucherVO) voucherList.getSelected();
				verVoucher(cc);
			}
		});
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void verVoucher(VoucherVO gc) {
		if (gc == null) {
			return;
		}

		Map args = new HashMap();
		args.put("voucherId", gc.getVoucherId());

		openWindow("/gui/configuraciones_comerciales/negcorporativos/editarVoucher.zul",
				Labels.getLabel("editarVoucherController.window.title"), args, MODAL);
	}

	public void onClick$btnLegalizar(Event ev) throws InterruptedException {
		try {
			validaCampos();
			
			executaLegalizacao();
			
		} catch (BusinessException bex) {
			Messagebox.show(
					bex.getMessage(),
					Labels.getLabel(TITULO),
					Messagebox.OK, Messagebox.ERROR);			
		} catch (Exception ex) {
			log.error(ex.getMessage());
			Messagebox.show(
						Labels.getLabel("MSG.Error"),
						Labels.getLabel(TITULO),
						Messagebox.OK, Messagebox.ERROR);
		}
		
	}
	
	public void onBlur$txtNit(Event ev) throws InterruptedException {
		if(txtNit.getValue() != null && !txtNit.getValue().isEmpty()) {
			Transportadora transp = transportadoraService.buscarPorNit(txtNit.getValue());
			if(transp != null) {
				selecionaCombo(transp, cmbTransportadora);
				txtNumContrato.setFocus(true);
			}else {
				Messagebox.show(
						Labels.getLabel("legalizacaoMassivaController.MSG.nitNaoEncontrado"),
						Labels.getLabel(TITULO),
						Messagebox.OK, Messagebox.ERROR);
			}
		}
	}
	
	private void executaLegalizacao() {
		
		Transportadora transportadora;		
		transportadora = (Transportadora)cmbTransportadora.getSelectedItem().getValue();
		
		Parada origem = null;
		Comboitem orig = cmbOrigem.getSelectedItem();		
		if( orig != null ) {
			origem = (Parada)orig.getValue();
		}
		
		Parada destino = null;
		Comboitem dest = cmbDestino.getSelectedItem();		
		if( dest != null ) {
			destino = (Parada)dest.getValue();
		}
		
		List<VoucherVO> processamento = voucherService.legalizar( txtNumInicial.getValue(),
									  txtNumFinal.getValue(),
									  txtNumContrato.getValue(),
									  transportadora,
									  txtValorLegalizado.getValue(),
									  origem, 
									  destino);

		voucherList.setData(processamento);
		preencheComplemento();
		pagingLegalizar.setVisible(true);
		voucherList.setVisible(true);
	}

	private void validaCampos() throws BusinessException {
		if ( txtNumInicial.getValue() == null 
				|| txtNumFinal.getValue() == null 
				|| txtValorLegalizado.getValue() == null  
				|| cmbTransportadora.getSelectedItem() == null ){
			throw new BusinessException("legalizacaoMassivaController.MSG.camposObrigatorios");
		}		
	}
	
	private void preencheComplemento() {
		for (Object item : voucherList.getListData()) {
			VoucherVO obj = (VoucherVO)item;
			if( StringUtils.isEmpty(obj.getDescOrigem()) || StringUtils.isEmpty(obj.getDescDestino() )) {
				List<String> origemDestino = paradaService.buscarDescOrigemDestino(obj.getOrigenId(), obj.getDestinoId());
				if(! origemDestino.isEmpty() ) {
					obj.setDescOrigem( origemDestino.get(0) );
					obj.setDescDestino( origemDestino.get(1) );
				}
			}
		}
	}	
}
